VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsSpell"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'*******************************************************************************
'
'  Spell Check VB Addin - Spell Check Comments and Strings in VB code.
'
'  Module:  clsspell.cls
'  Author:  Dean J. Giovanelli (deangio.pa@netzero.net)
'  Date:    12/05/2000
'
'  Description:
'    Somewhat generic Spell checker object that could be reused - however
'    its main purpose is to isolate where changes need to be made to
'    use other spell checkers like MS-Excel and internationalization changes.
'    If you reuse this class elsewhere remember to create utlErrorHandler().
'
'  Modifications:
'   Help hungry children: How you treat the poorest - is how you treat God.
'*******************************************************************************
Option Explicit

' When True the users Custom Dictionary is ignored, False= its not ignored.
Private mbUseMainDictionaryOnly As Boolean

' Name of the Custom Dictionary to use. Notice that only 1 is provided
Private msNameOfCustomDict As String

' Name of the Dictionary used to store Added Words
Private msNameOfAddWordsDict As String

' List of words to "Ignore All"
Private mobjCurrentlyIgnoredWords As Collection

' List of words to "Change All" values to
Private mobjCurrentlyChangedAllWords As Collection

' Word application
Private mobjWordApplication As Word.Application
' Word application suggestions object
Private mobjSuggestions As Word.SpellingSuggestions
' Word application suggestion object
Private objSuggestion As Word.SpellingSuggestion

'-------------------------------------------------------------------------------
'
' SpellCheckWord - Spell Check the given word
'
' Parameters:
'   pWord            -  I: Word to lookup
'   psSuggestedWords -  O: Collection of suggested words (if Word is misspelled)
'   pSuggestionsCnt  -  O: Number of Suggested words
'
' Returns:
'   True=Word is legal, False=Word is misspelled
'
' Comments:
'   = An important part of the design of this Addin to to delay as long as
'     possible starting up a word object (which is time intensive) because
'     if the user loads this addin at every startup we don't want to delay vb.
'     Don't try to cheat and use GetObject() hoping to get an instance to save
'     time because there are several problems that can cause.
'
'-------------------------------------------------------------------------------
Public Function SpellCheckWord(ByVal pWord As String, psSuggestedWords() As String, _
    pSuggestionsCnt As Long)
    
    On Error GoTo Proc_Err
    Dim llsize As Long
    Dim lbDidItWork As Boolean
    Static lHaveIBeenCalledBefore As Boolean
    
    ' Allocate word if we have not done so already
    If lHaveIBeenCalledBefore = False Then
        ' Make sure if we already allocated that we don't do it again.
        Set mobjWordApplication = Nothing
        ' We need to create a new word object
        Set mobjWordApplication = New Word.Application
        ' Make sure the app is not visible - so the user cant quit it.
        mobjWordApplication.Visible = False
        lHaveIBeenCalledBefore = True
    End If
    
    ' One document is required to get suggestions
    If mobjWordApplication.Documents.Count = 0 Then
        mobjWordApplication.Documents.Add
    End If
    
    ' Check the words spelling...
    If Len(msNameOfCustomDict) = 0 Then
        lbDidItWork = mobjWordApplication.CheckSpelling(pWord, msNameOfAddWordsDict)
    Else
        If mbUseMainDictionaryOnly = False Then
        ' We must include the custom dictionary
        lbDidItWork = mobjWordApplication.CheckSpelling(pWord, msNameOfAddWordsDict, , , msNameOfCustomDict)
        End If
    End If
    If lbDidItWork Then
        ' Word is found - so MSWord thinks its spelled legally.
        SpellCheckWord = True
    Else
        ' Word is not found - so get the suggestions
        SpellCheckWord = False
        Set mobjSuggestions = Nothing
        Set mobjSuggestions = mobjWordApplication.GetSpellingSuggestions(pWord)
        pSuggestionsCnt = CLng(mobjSuggestions.Count)
        If pSuggestionsCnt > 0 Then
            llsize = 0
            ' Fine to use redim because there should only be less than 75 suggestions.
            ReDim Preserve psSuggestedWords(CLng(pSuggestionsCnt))
            For Each objSuggestion In mobjSuggestions
                llsize = llsize + 1
                psSuggestedWords(llsize) = CStr(objSuggestion.Name)
            Next
        End If
    End If
    
Proc_Exit:
    Exit Function

Proc_Err:
    utlErrorHandler "SpellCheckWord"
    Resume Proc_Exit
End Function

'-------------------------------------------------------------------------------
'
' Class_Terminate - Cleanup memory
'
' Parameters:
'   None.
'
' Returns:
'   None.
'
' Comments:
'   None.
'
'-------------------------------------------------------------------------------
Private Sub Class_Terminate()
    On Error Resume Next
    Set objSuggestion = Nothing
    Set mobjSuggestions = Nothing
    ' Must call quit method because setting object ref to Nothing will not
    ' destroy the object.
    mobjWordApplication.Quit False
    Set mobjWordApplication = Nothing
End Sub

'-------------------------------------------------------------------------------
'
' ClearIgnoredWords - Clear the one list of words to Ignore in this session
'
' Parameters:
'   None.
'
' Returns:
'   None.
'
' Comments:
'   See IsWordIgnored() and AddWordToIgnoredList()
'
'-------------------------------------------------------------------------------
Public Sub ClearIgnoredWords()
    On Error GoTo Proc_Err
    ' Destroy the old collection - if it exists
    Set mobjCurrentlyIgnoredWords = Nothing
    ' Create a new collection
    Set mobjCurrentlyIgnoredWords = New Collection
Proc_Exit:
    Exit Sub
Proc_Err:
    utlErrorHandler "ClearIgnoredWords"
    Resume Proc_Exit
End Sub

'-------------------------------------------------------------------------------
'
' IsWordIgnored - Returns True if the provided word is in the collection
'
' Parameters:
'   pWord - I: Word to lookup
'
' Returns:
'   Boolean: True=Yes, False=No
'
' Comments:
'   See ClearIgnoredWords() and AddWordToIgnoredList()
'
'-------------------------------------------------------------------------------
Public Function IsWordIgnored(pWord As String) As Boolean
    On Error GoTo Proc_Err
    If mobjCurrentlyIgnoredWords Is Nothing Then Exit Function
    If mobjCurrentlyIgnoredWords.Count = 0 Then
        IsWordIgnored = False
    Else
        If Len(mobjCurrentlyIgnoredWords(pWord)) > 0 Then
            ' Word is in the list of ignored words
            IsWordIgnored = True
        Else
            IsWordIgnored = False
        End If
    End If
Proc_Exit:
    Exit Function
Proc_Err:
    If Err.Number = 5 Then
        IsWordIgnored = False
        Resume Proc_Exit
    End If
    utlErrorHandler "IsWordIgnored"
    Resume Proc_Exit
End Function

'-------------------------------------------------------------------------------
'
' AddWordToIgnoredList - Adds the given word to the list
'
' Parameters:
'   pWord - I: Word to lookup
'
' Returns:
'   None.
'
' Comments:
'   = If its already there it is NOT added again.
'
'-------------------------------------------------------------------------------
Public Sub AddWordToIgnoredList(pWord As String)
    On Error GoTo Proc_Err
    If mobjCurrentlyIgnoredWords Is Nothing Then Exit Sub
    If IsWordIgnored(pWord) = False Then
        ' This word is not already ignored - so add it to the list
        mobjCurrentlyIgnoredWords.Add pWord, pWord
    End If
Proc_Exit:
    Exit Sub
Proc_Err:
    utlErrorHandler "AddWordToIgnoredList"
    Resume Proc_Exit
End Sub

'-------------------------------------------------------------------------------
'
' ClearChangedWords - Clear the one list of words to Change automatically
'   in this session
'
' Parameters:
'   None.
'
' Returns:
'   None.
'
' Comments:
'   See IsWordChanged() and AddWordToChangedList()
'
'-------------------------------------------------------------------------------
Public Sub ClearChangedWords()
    On Error GoTo Proc_Err
    ' Destroy the old collection - if it exists
    Set mobjCurrentlyChangedAllWords = Nothing
    ' Create a new collection
    Set mobjCurrentlyChangedAllWords = New Collection
Proc_Exit:
    Exit Sub
Proc_Err:
    utlErrorHandler "ClearChangedWords"
    Resume Proc_Exit
End Sub

'-------------------------------------------------------------------------------
'
' IsWordChanged - Returns True if the provided word is in the changed collection
'
' Parameters:
'   pWord - I: Word to lookup
'
' Returns:
'   Boolean: True=Yes, False=No
'
' Comments:
'   See ClearChangedWords() and AddWordToChangedList()
'
'-------------------------------------------------------------------------------
Public Function IsWordChanged(pWord As String) As Boolean
    On Error GoTo Proc_Err
    If mobjCurrentlyChangedAllWords Is Nothing Then Exit Function
    If mobjCurrentlyChangedAllWords.Count = 0 Then
        IsWordChanged = False
    Else
        If Len(mobjCurrentlyChangedAllWords(pWord)) > 0 Then
            ' Yes word is here so its in the changes List
            IsWordChanged = True
        Else
            IsWordChanged = False
        End If
    End If
Proc_Exit:
    Exit Function
Proc_Err:
    If Err.Number = 5 Then
        IsWordChanged = False
        Resume Proc_Exit
    End If
    utlErrorHandler "IsWordChanged"
    Resume Proc_Exit
End Function

'-------------------------------------------------------------------------------
'
' AddWordToChangedList - Adds the given word to the list
'
' Parameters:
'   pWord - I: Word to lookup
'
' Returns:
'   None.
'
' Comments:
'   = If its already there it is NOT added again.
'
'-------------------------------------------------------------------------------
Public Sub AddWordToChangedList(pWord As String, pCorrectedWord As String)
    On Error GoTo Proc_Err
    If mobjCurrentlyChangedAllWords Is Nothing Then Exit Sub
    If IsWordChanged(pWord) = False Then
        ' This word is not already ignored - so add it to the list
        mobjCurrentlyChangedAllWords.Add pCorrectedWord, pWord
    End If
Proc_Exit:
    Exit Sub
Proc_Err:
    utlErrorHandler "AddWordToChangedList"
    Resume Proc_Exit
End Sub

'-------------------------------------------------------------------------------
'
' GetChangedWord - Return the word that pWord is replaced with
'
' Parameters:
'   None.
'
' Returns:
'   String of replacement word, or "" if that word does not exist
'
' Comments:
'   None.
'
'-------------------------------------------------------------------------------
Public Function GetChangedWord(pWord As String) As String
    On Error GoTo Proc_Err
    If mobjCurrentlyChangedAllWords Is Nothing Then
        ' Do nothing
    Else
        GetChangedWord = mobjCurrentlyChangedAllWords.Item(pWord)
    End If
Proc_Exit:
    Exit Function
Proc_Err:
    If Err.Number = 5 Then
        GetChangedWord = ""
        Resume Proc_Exit
    End If
    utlErrorHandler "GetChangedWord"
    Resume Proc_Exit
End Function

'-------------------------------------------------------------------------------
'
' DoesFileExists - Check to see if the file exists
'
' Parameters:
'   pFileName - I: FullPath to check
'
' Returns:
'   True=File Exists, False=It doesn't
'
' Comments:
'   None.
'
'-------------------------------------------------------------------------------
Public Function DoesFileExists(ByVal pFileName As String) As Boolean
    On Error GoTo Proc_Exit
    Dim liFileId As Integer
    liFileId = FreeFile
    ' Can we open it?
    Open pFileName For Input Access Read As liFileId
    Close #liFileId
    If Len(Dir(pFileName)) = 0 Then GoTo Proc_Exit
    DoesFileExists = True
    Exit Function
Proc_Exit:
    DoesFileExists = False
End Function

'-------------------------------------------------------------------------------
'
' AppendToAddWordsFile - Append a new line in the Add words Textfile
'
' Parameters:
'   pFileName - I: FileName to append
'   pString -   I: String to write
'
' Returns:
'   None.
'
' Comments:
'   = Add words is done via adding words to a Word dictionary. This function
'     really just adds a new word and does NOT check for duplicates which work
'     ok anyway.
'
'-------------------------------------------------------------------------------
Public Sub AppendToAddWordsFile(ByVal pFileName As String, pString As String)
    
    On Error GoTo Proc_Err
    Dim lsCompleteFileName As String
    Dim pFileNum As Integer ' File descriptor

    ' Add the fullpath if the filename doesn't include it
    If InStr(pFileName, "\") < 1 Then
        lsCompleteFileName = CurDir + "\" + pFileName
    Else
        lsCompleteFileName = pFileName
    End If
    
    ' Get the next available file descriptor
    pFileNum = FreeFile
    
    ' Open the file in Append mode (even if it does not exists)
    On Error GoTo BadFileName
    Open lsCompleteFileName For Append As #pFileNum
    On Error GoTo Proc_Err
    
    ' Write the message
    Print #pFileNum, pString
    
    ' Close the file
    Close #pFileNum
Proc_Exit:
    Exit Sub

BadFileName:
    ' User gave an invalid filename so just quietly use the default
    lsCompleteFileName = "C:\MyVbAddedWords.txt"
    Resume
    
Proc_Err:
    utlErrorHandler "AppendToAddWordsFile"
    Resume Proc_Exit
End Sub

'-------------------------------------------------------------------------------
'
' NameOfCustomDict - Get the name of the Custom Dictionary being used
'
' Parameters:
'   None.
'
' Returns:
'   None.
'
' Comments:
'   None.
'
'-------------------------------------------------------------------------------
Public Property Get NameOfCustomDict() As String
    On Error GoTo Proc_Err
    NameOfCustomDict = msNameOfCustomDict
Proc_Exit:
    Exit Property
Proc_Err:
    utlErrorHandler "NameOfCustomDict"
    Resume Proc_Exit
End Property

'-------------------------------------------------------------------------------
'
' NameOfCustomDict - Set the name of the custom Dictionary being used
'
' Parameters:
'   vNewValue -
'
' Returns:
'   None.
'
' Comments:
'   None.
'
'-------------------------------------------------------------------------------
Public Property Let NameOfCustomDict(ByVal vNewValue As String)
    On Error GoTo Proc_Err
    msNameOfCustomDict = vNewValue
Proc_Exit:
    Exit Property
Proc_Err:
    utlErrorHandler "NameOfCustomDict"
    Resume Proc_Exit
End Property

'-------------------------------------------------------------------------------
'
' UseMainDictionaryOnly - Get value of UseMainDictionaryOnly
'
' Parameters:
'   None.
'
' Returns:
'   None.
'
' Comments:
'   None.
'
'-------------------------------------------------------------------------------
Public Property Get UseMainDictionaryOnly() As Boolean
    On Error GoTo Proc_Err
    UseMainDictionaryOnly = mbUseMainDictionaryOnly
Proc_Exit:
    Exit Property
Proc_Err:
    utlErrorHandler "UseMainDictionaryOnly"
    Resume Proc_Exit
End Property

'-------------------------------------------------------------------------------
'
' UseMainDictionaryOnly - Set value of UseMainDictionaryOnly
'
' Parameters:
'   vNewValue -
'
' Returns:
'   None.
'
' Comments:
'   None.
'
'-------------------------------------------------------------------------------
Public Property Let UseMainDictionaryOnly(ByVal vNewValue As Boolean)
    On Error GoTo Proc_Err
    mbUseMainDictionaryOnly = vNewValue
Proc_Exit:
    Exit Property
Proc_Err:
    utlErrorHandler "UseMainDictionaryOnly"
    Resume Proc_Exit
End Property

'-------------------------------------------------------------------------------
'
' NameOfAddWordsDict - Get value of NameOfAddWordsDict
'
' Parameters:
'   None.
'
' Returns:
'   None.
'
' Comments:
'   None.
'
'-------------------------------------------------------------------------------
Public Property Get NameOfAddWordsDict() As String
    On Error GoTo Proc_Err
    NameOfAddWordsDict = msNameOfAddWordsDict
Proc_Exit:
    Exit Property
Proc_Err:
    utlErrorHandler "NameOfAddWordsDict"
    Resume Proc_Exit
End Property

'-------------------------------------------------------------------------------
'
' NameOfAddWordsDict - Set value of NameOfAddWordsDict
'
' Parameters:
'   vNewValue -
'
' Returns:
'   None.
'
' Comments:
'   None.
'
'-------------------------------------------------------------------------------
Public Property Let NameOfAddWordsDict(ByVal vNewValue As String)
    On Error GoTo Proc_Err
    msNameOfAddWordsDict = vNewValue
Proc_Exit:
    Exit Property
Proc_Err:
    utlErrorHandler "NameOfAddWordsDict"
    Resume Proc_Exit
End Property
