Attribute VB_Name = "basMain"
'*******************************************************************************
'
'  Spell Check VB Addin - Spell Check Comments and Strings in VB code.
'
'  Module:  basmain.bas
'  Author:  Dean J. Giovanelli (deangio.pa@netzero.net)
'  Date:    12/05/2000
'
'  Description:
'    Main module used for public variables and public constants. Also has
'    simple global error handling.
'
'  Modifications:
'   Help hungry children: How you treat the poorest - is how you treat God.
'*******************************************************************************
Option Explicit

' Used to start WordPad
Private Declare Function ShellExecute Lib "shell32.dll" _
    Alias "ShellExecuteA" (ByVal hWnd As Long, ByVal lpOperation As String, _
    ByVal lpFile As String, ByVal lpParameters As String, _
    ByVal lpDirectory As String, ByVal nShowCmd As Long) As Long
'
' Global variables
'
' Application reference - do not store in the Connect designer
Public oVBE As vbide.VBE
' Add instance
Public gAddInInst As Object
'docMain ToolWindow
Public gspellWindow As vbide.Window
Public aiSpellMsg As Object
'
' Global constants
'
' Title used in all MsgBox calls
Public Const ADDIN_TITLE As String = "Spell Check VB Addin"
' Reference of the bitmap in the RES file.
Public Const MAIN_BITMAP_IN_RESFILE As Integer = 101
' Default dictionary that holds our added words
Public Const DEFAULTADDWORDDICT As String = "TempAddinWords.dic"

'-------------------------------------------------------------------------------
'
' utlErrorHandler - Very simple global Error Handler
'
' Parameters:
'   pErrorModule -  I: Module name containing the error
'   pOtherInfo -    I: (optional) Any additional text appearing after the
'                      err.description
'
' Returns:
'   None.
'
' Comments:
'   = No fancy error handling to keep this code basic.
'
'-------------------------------------------------------------------------------
Public Sub utlErrorHandler(ByVal pErrorModule As String, _
    Optional ByVal pOtherInfo As String = "")
    Dim llErrorNumber As Long
    Dim lsErrorText As String
    
    ' Setup
    llErrorNumber = Err.Number
    lsErrorText = Err.Description & ":" & pOtherInfo
    ' Show Critical MsgBox
    MsgBox "Unexpected Error in Spell Check VB Addin" & vbCrLf & _
           "Procedure  = " & pErrorModule & vbCrLf & _
           "Err.Number     = " & llErrorNumber & vbCrLf & _
           "Err.Description= " & lsErrorText, vbCritical, ADDIN_TITLE
End Sub

'-------------------------------------------------------------------------------
'
' ShellProgram - A better version of Shell() with error control
'
' Parameters:
'   psFilePath    - I: Program to start
'   psParms       - I: command line args
'   psDir         - I: Directory
'   psDisplayName - I: Name of program as appearing in any error messages
'
' Returns:
'   True=it worked, False=Error
'
' Comments:
'   None.
'
'-------------------------------------------------------------------------------
Public Function ShellProgram(ByVal psFilePath As String, _
    ByVal psParms As String, ByVal psDir As String, psDisplayName As String) As Boolean
    
    On Error GoTo Proc_Err
    Dim lsMsg As String
    Dim lsErr As String
    Dim lihwndProgram As Integer
    lsErr = ""
    lihwndProgram = ShellExecute(0, "Open", psFilePath, psParms, psDir, 1)
    Select Case (lihwndProgram)
        Case 0: lsErr = "Insufficient memory or corrupt"
        Case 2: lsErr = "File not found."
        Case 3: lsErr = "Invalid path."
        Case 5: lsErr = "Sharing or Protection Error."
        Case 6: lsErr = "Separate data segments are required for each task."
        Case 8: lsErr = "Insufficient memory to execute."
        Case 10: lsErr = "Invalid Windows version."
        Case 11: lsErr = "Invalid program file."
        Case 12: lsErr = "requires a different OS."
        Case 13: lsErr = "requires a different OS."
        Case 14: lsErr = "is an unknown program file type."
        Case 15: lsErr = "Protected memory mode problem"
        Case 16: lsErr = "Invalid use of data segments when loading a second instance of"
        Case 19: lsErr = "is a compressed program file."
        Case 20: lsErr = "Invalid DLL."
        Case 21: lsErr = "requires Windows 32-bit extensions."
    End Select
    '
    If Len(lsErr) = 0 Then
        ShellProgram = True
    Else
        lsMsg = "Unable to Start " & psDisplayName & ": " & lsErr
        MsgBox lsMsg, vbCritical, ADDIN_TITLE & " :" & psDisplayName
        ShellProgram = False
    End If
Proc_Exit:
    Exit Function
Proc_Err:
    utlErrorHandler "ShellProgram"
    Resume Proc_Exit
End Function
