VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsAddin"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'*******************************************************************************
'
'  Spell Check VB Addin - Spell Check Comments and Strings in VB code.
'
'  Module:  clsaddin.cls
'  Author:  Dean J. Giovanelli (deangio.pa@netzero.net)
'  Date:    12/05/2000
'
'  Description:
'    Class of generic functions used in Addin programming I wish MS provided.
'    If you reuse this class elsewhere remember to create utlErrorHandler()
'    and include the dependant clsString object.
'
'  Modifications:
'   Help hungry children: How you treat the poorest - is how you treat God.
'*******************************************************************************
Option Explicit
' Reference to our private string object
Private mobjStr As clsString

'-------------------------------------------------------------------------------
'
' GetCurrentModuleAsString - Get a string made up of all text in the current
'                            (active) Module and the starting & ending lines
'
' Parameters:
'   pStartOfProc -  O: Starting line of the Module (Always 1)
'   pEndOfProc   -  O: Ending line of the Module
'
' Returns:
'   String or "" if there is not an active code pane.
'
' Comments:
'   = See related GetCurrentProcAsString()
'
'-------------------------------------------------------------------------------
Public Function GetCurrentModuleAsString(ByRef pStartOfProc As Long, _
    ByRef pEndOfProc As Long) As String
    On Error GoTo Proc_Err
    If oVBE.ActiveCodePane Is Nothing Then
        ' There is no ActiveCodePane so return blank
        GetCurrentModuleAsString = ""
        pStartOfProc = 0
        pEndOfProc = 0
    Else
        pStartOfProc = 1
        ' Get all the lines into a string
        With oVBE.ActiveCodePane.CodeModule
            ' Get the Total Number of Lines in the CodeModule
            pEndOfProc = .CountOfLines
            GetCurrentModuleAsString = .Lines(1, pEndOfProc)
        End With
    End If
Proc_Exit:
    Exit Function
Proc_Err:
    utlErrorHandler "GetCurrentModuleAsString"
    Resume Proc_Exit
End Function

'-------------------------------------------------------------------------------
'
' GetCurrentProcAsString - Get a string made up of all text in the
'                          Procedure (or General Declares) at the given line.
'
' Parameters:
'   pStartLine   -  I: Line in a Procedure or General Declares section.
'   pStartOfProc -  O: Starting line of the Procedure (or General Declares)
'   pEndOfProc   -  O: Ending line of the Procedure (or General Declares)
'
' Returns:
'   String or "" if there is not an active code pane.
'
' Comments:
'   = We always treat the General Declares section as its own procedure.
'
'-------------------------------------------------------------------------------
Public Function GetCurrentProcAsString(ByVal pStartLine As Long, _
    ByRef pStartOfProc As Long, ByRef pEndOfProc As Long) As String
    On Error GoTo Proc_Err
    Dim lobjMember   As vbide.Member
    Dim lsNameOfMethodOrProperty As String
    Dim liProcKind As Integer
    Dim llPropGetCount As Long
    Dim llPropGetStart As Long
    Dim llPropLetCount As Long
    Dim llPropLetStart As Long
    Dim llPropSetCount As Long
    Dim llPropSetStart As Long
    Dim lNumOfDecLines As Long
    Dim lFirstCodeLine As Long
    
    If oVBE.ActiveCodePane Is Nothing Then
        ' There is no ActiveCodePane so return blank
        GetCurrentProcAsString = ""
        pStartOfProc = 0
        pEndOfProc = 0
    Else
        With oVBE.ActiveCodePane.CodeModule
        ' Get the count of declaration lines
        lNumOfDecLines = .CountOfDeclarationLines
        ' Locate the first line of code
        lFirstCodeLine = lNumOfDecLines + 1
            
        ' Determine if we are in the General Declares section or in code
        If pStartLine < lFirstCodeLine Then
            ' We are in the General Declares section so instead treat the General Declares section
            ' as the current procedure
                pStartOfProc = 1
                pEndOfProc = lNumOfDecLines
                GetCurrentProcAsString = .Lines(pStartOfProc, pEndOfProc)
            Else ' Normal
                ' Figure out the name of our procedure or property - yes this does work for Properties.
                lsNameOfMethodOrProperty = .ProcOfLine(pStartLine, vbext_pk_Proc)
                ' Find our procedure name in the Members collection
                Set lobjMember = .Members(lsNameOfMethodOrProperty)
                ' Determine if we are a Procedure or a Property
                If lobjMember.Type = vbext_mt_Method Then
                    ' Our object is a member
                    pStartOfProc = .ProcStartLine(lsNameOfMethodOrProperty, vbext_pk_Proc)
                    pEndOfProc = pStartOfProc + _
                        .ProcCountLines(lsNameOfMethodOrProperty, vbext_pk_Proc) - 1
                    ' Get the whole Procedure as a string
                    GetCurrentProcAsString = .Lines(pStartOfProc, _
                        .ProcCountLines(lsNameOfMethodOrProperty, vbext_pk_Proc))
                ElseIf lobjMember.Type = vbext_mt_Property Then
                    ' Extract the property values - ignore exceptions
                    On Error Resume Next
                    llPropGetCount = .ProcCountLines(lsNameOfMethodOrProperty, vbext_pk_Get)
                    llPropGetStart = .ProcStartLine(lsNameOfMethodOrProperty, vbext_pk_Get)
                    llPropLetCount = .ProcCountLines(lsNameOfMethodOrProperty, vbext_pk_Let)
                    llPropLetStart = .ProcStartLine(lsNameOfMethodOrProperty, vbext_pk_Let)
                    llPropSetCount = .ProcCountLines(lsNameOfMethodOrProperty, vbext_pk_Set)
                    llPropSetStart = .ProcStartLine(lsNameOfMethodOrProperty, vbext_pk_Set)
                    On Error GoTo Proc_Err
                    
                    ' Determine which of the three possible ranges our line falls within
                    If pStartLine >= llPropGetStart And _
                        pStartLine < (llPropGetStart + llPropGetCount) Then
                        liProcKind = vbext_pk_Get
                        pStartOfProc = llPropGetStart
                        pEndOfProc = pStartOfProc + llPropGetCount - 1
                    ElseIf pStartLine >= llPropLetStart And _
                        pStartLine < (llPropLetStart + llPropLetCount) Then
                        liProcKind = vbext_pk_Let
                        pStartOfProc = llPropLetStart
                        pEndOfProc = pStartOfProc + llPropLetCount - 1
                    ElseIf pStartLine >= llPropSetStart And _
                        pStartLine < (llPropSetStart + llPropSetCount) Then
                        liProcKind = vbext_pk_Set
                        pStartOfProc = llPropSetStart
                        pEndOfProc = pStartOfProc + llPropSetCount - 1
                    End If
                    ' Get the whole Property as a string
                    GetCurrentProcAsString = .Lines(pStartOfProc, .ProcCountLines(lsNameOfMethodOrProperty, liProcKind))
                End If
            End If
        End With
    End If
    
Proc_Exit:
    Exit Function

Proc_Err:
    utlErrorHandler "GetCurrentProcAsString"
    Resume Proc_Exit
End Function

'-------------------------------------------------------------------------------
'
' Class_Initialize - Create classes dependant objects
'
' Parameters:
'   None.
'
' Returns:
'   None.
'
' Comments:
'   None.
'
'-------------------------------------------------------------------------------
Private Sub Class_Initialize()
    On Error GoTo Proc_Err
    ' Create our private string object
    Set mobjStr = New clsString
Proc_Exit:
    Exit Sub
Proc_Err:
    utlErrorHandler "clsAddin.Class_Initialize"
    Resume Proc_Exit
End Sub

'-------------------------------------------------------------------------------
'
' Class_Terminate - Cleanup classes dependant objects
'
' Parameters:
'   None.
'
' Returns:
'   None.
'
' Comments:
'   None.
'
'-------------------------------------------------------------------------------
Private Sub Class_Terminate()
    On Error Resume Next
    ' Destroy our private string object
    Set mobjStr = Nothing
End Sub

'-------------------------------------------------------------------------------
'
' ReplaceOneLineInCode - Replace the line at the given position, with the
'                        pNewLine in the ActiveCodePane.
'
' Parameters:
'   pLine    -  I: Line position
'   pNewLine -  I: String to place at pLine
'
' Returns:
'   True=ok, False=Failed
'
' Comments:
'   = Just a wrapper around .ReplaceLine()
'
'-------------------------------------------------------------------------------
Public Function ReplaceOneLineInCode(ByVal pLine As Long, _
    ByVal pNewLine As String) As Boolean
    On Error GoTo Proc_Err
    Dim lTotalLinesInModule As Long
    If oVBE.ActiveCodePane Is Nothing Then
        ' There is no ActiveCodePane so return False
        ReplaceOneLineInCode = False
    Else
        With oVBE.ActiveCodePane.CodeModule
            lTotalLinesInModule = .CountOfLines
            If pLine > lTotalLinesInModule Or pLine < 1 Then
                ' pLine doesn't exist in the code module
                ReplaceOneLineInCode = False
            Else
                ' Replace the line
                .ReplaceLine pLine, pNewLine
                ReplaceOneLineInCode = True
            End If
        End With
    End If
Proc_Exit:
    Exit Function
Proc_Err:
    utlErrorHandler "ReplaceOneLineInCode"
    Resume Proc_Exit
End Function

'-------------------------------------------------------------------------------
'
' GetCurrentLinePosition - Get the Current Line in the CodePane
'
' Parameters:
'   None.
'
' Returns:
'   Line number in the ActiveCodePane
'
' Comments:
'   = Very simple wrapper function around .GetSelection
'
'-------------------------------------------------------------------------------
Public Function GetCurrentLinePosition() As Long
    On Error GoTo Proc_Err
    Dim lStartLine As Long
    Dim lStartCol As Long
    Dim lEndLine As Long
    Dim lEndCol As Long
    If oVBE.ActiveCodePane Is Nothing Then
        ' There is no ActiveCodePane so return 0
        GetCurrentLinePosition = 0
    Else
        ' Get the Line
        oVBE.ActiveCodePane.GetSelection lStartLine, lStartCol, lEndLine, lEndCol
        GetCurrentLinePosition = lStartLine
    End If
Proc_Exit:
    Exit Function
Proc_Err:
    utlErrorHandler "GetCurrentLinePosition"
    Resume Proc_Exit
End Function

'-------------------------------------------------------------------------------
'
' GetSelectedTextAsString - Get the selected text as a string and its coords.
'
' Parameters:
'   pStartLine -    O: Start Line
'   pStartCol  -    O: Start Column
'   pEndLine   -    O: End Line
'   pEndCol    -    O: End Column
'
' Returns:
'   String with the selected text
'
' Comments:
'   None.
'
'-------------------------------------------------------------------------------
Public Function GetSelectedTextAsString(ByRef pStartLine As Long, _
    ByRef pStartCol As Long, ByRef pEndLine As Long, _
    ByRef pEndCol As Long) As String

    On Error GoTo Proc_Err

    Dim lStartLine As Long
    Dim lStartCol As Long
    Dim lEndLine As Long
    Dim lEndCol As Long
    Dim lsTarget As String
    Dim lsFirstLine As String
    Dim lsMiddle As String
    Dim lsLastLine As String
    Dim lli As Long
    
    If oVBE.ActiveCodePane Is Nothing Then
        ' There is no ActiveCodePane so return 0
        GetSelectedTextAsString = ""
    Else
        ' Get the Line
        With oVBE.ActiveCodePane.CodeModule
            oVBE.ActiveCodePane.GetSelection lStartLine, lStartCol, lEndLine, lEndCol
            pStartLine = lStartLine
            pStartCol = lStartCol
            pEndLine = lEndLine
            pEndCol = lEndCol
            
            If lStartLine = lEndLine Then
                ' There is only one line to get the text from
                If lEndCol > lStartCol Then
                    lsTarget = .Lines(lStartLine, 1)
                    lsTarget = Mid$(lsTarget, lStartCol, lEndCol - lStartCol)
                End If
            Else
                ' Get the First Line
                lsFirstLine = .Lines(lStartLine, 1)
                lsFirstLine = Mid$(lsFirstLine, lStartCol)
                lsFirstLine = lsFirstLine & vbCrLf
                ' Get the Last Line
                lsLastLine = .Lines(lEndLine, 1)
                lsLastLine = Left$(lsLastLine, lEndCol - 1) ' The -1 is because lEndCol is off by one
                ' Now Get all the lines in-between
                If lEndLine > (lStartLine + 1) Then
                    ' There is at least one middle line
                    For lli = lStartLine + 1 To lEndLine - 1
                        lsMiddle = lsMiddle & .Lines(lli, 1)
                        lsMiddle = lsMiddle & vbCrLf
                    Next
                    lsTarget = lsFirstLine & lsMiddle & lsLastLine
                Else
                    ' There was two lines selected
                    lsTarget = lsFirstLine & lsLastLine
                End If
            End If
            GetSelectedTextAsString = lsTarget
        End With
    End If

Proc_Exit:
    Exit Function

Proc_Err:
    utlErrorHandler "GetSelectedTextAsString"
    Resume Proc_Exit
End Function

'-------------------------------------------------------------------------------
'
' SelectWordInCodePane - Select a word at the given location
'
' Parameters:
'   pWord - I: Word (which may or may not be there)
'   pLine - I: Line
'   pCol  - I: Col
'
' Returns:
'   None.
'
' Comments:
'   None.
'
'-------------------------------------------------------------------------------
Public Sub SelectWordInCodePane(ByVal pWord As String, ByVal pLine As Long, ByVal pCol As Long)
    On Error GoTo Proc_Err
    If oVBE.ActiveCodePane Is Nothing Then
        ' There is no ActiveCodePane so ignore the request
    Else
        oVBE.ActiveCodePane.SetSelection pLine, pCol, pLine, (pCol + Len(pWord))
    End If
Proc_Exit:
    Exit Sub
Proc_Err:
    utlErrorHandler "SelectWordInCodePane"
    Resume Proc_Exit
End Sub

'-------------------------------------------------------------------------------
'
' SetFocusToActiveCodeWindow - Set Focus to the ActiveCodePane in vb
'
' Parameters:
'   None.
'
' Returns:
'   None.
'
' Comments:
'   = If this is not done you will not see the word selected in the CodePane
'
'-------------------------------------------------------------------------------
Public Sub SetFocusToActiveCodeWindow()
    On Error GoTo Proc_Err
    If oVBE.ActiveCodePane Is Nothing Then
        ' Cant do it now
        GoTo Proc_Exit
    Else
        ' Set the Focus
        oVBE.ActiveCodePane.Window.SetFocus
    End If
Proc_Exit:
    Exit Sub
Proc_Err:
    utlErrorHandler "SetFocusToActiveCodeWindow"
    Resume Proc_Exit
End Sub

'-------------------------------------------------------------------------------
'
' GetCodeLineInString - Get a line of code starting at the given position
'
' Parameters:
'   pSource   - I: Source
'   pLineNum  - I: Line (starting at 1)
'   pStartCol - I: Col (starting at 1)
'
' Returns:
'   String requested or "" when invalid
'
' Comments:
'   None.
'
'-------------------------------------------------------------------------------
Public Function GetCodeLineInString(pSource As String, ByVal pLineNum As Long, _
    ByVal pStartCol As Long) As String
    
    On Error GoTo Proc_Err
    Dim lPosStart As Long
    Dim lEndPos As Long
    Dim lli As Long
    Dim lsLine As String
    
    ' Skip over the over lines - not really efficient - but it works.
    lPosStart = 1
    For lli = 1 To pLineNum - 1
        lPosStart = InStr(lPosStart, pSource, vbCrLf)
        If lPosStart < 1 Then
            GetCodeLineInString = ""
            GoTo Proc_Exit
        End If
        ' Point past the least vbcrlf
        lPosStart = lPosStart + 1
    Next
    
    ' Find the end of the requested line
    lEndPos = InStr(lPosStart, pSource, vbCrLf)
    
    ' Form the line
    If lEndPos = lPosStart Then
        ' This happens when a return is the only char on the line
        GetCodeLineInString = " "
        GoTo Proc_Exit
    ElseIf lEndPos = 0 Then
        ' Last line
        lsLine = Mid$(pSource, lPosStart)
    Else
        lsLine = Mid$(pSource, lPosStart, lEndPos - lPosStart)
    End If
    ' Strip out any control characters in the line.
    lsLine = mobjStr.StripCtrlCharsInStr(Trim$(lsLine))
    
    ' Modify the line to make use of the pStartCol
    If pStartCol <= Len(lsLine) Then
        ' Adjust the line to make use of the Starting position
        lsLine = Mid$(lsLine, pStartCol)
    End If
    GetCodeLineInString = lsLine
    
Proc_Exit:
    Exit Function

Proc_Err:
    utlErrorHandler "GetCodeLineInString"
    Resume Proc_Exit
End Function

'-------------------------------------------------------------------------------
'
' GetPositionsToCheckInLine - Get a collection containing the Positions in the
'                             pLine we want to look at based on the pMethod
'
' Parameters:
'   pLine               -   I:  Line of Code
'   PositionsCollection -   IO: Collection to of character ranges to check based
'                               on pMethod
'   pMethod             -   I:  1 = Comment-Get up to one range containing the
'                                   VB comments
'                               2 = String-Get ranges of strings in code.
'                               3=  Both-Get ranges of strings in code &
'                                   possibly one range of comments
' Returns:
'   None.
'
' Comments:
'   The purpose is to return a collection which contains multiple starting
'   and ending ranges of columns in the line that we are interested in checking.
'   E.g. Using pMethod=3
'       Debug.Print "Hello" & cstr(a) & "Smith" ' Say Hello
'   Produces 3 elements in the collection... Where "S"=String, "C"=Comment
'       "S14,18", "S34,38", "C42,51"
'   The key field in the collection starts 0 to Count-1
'-------------------------------------------------------------------------------
Public Sub GetPositionsToCheckInLine(pLine As String, PositionsCollection As Collection, _
    ByVal pMethod As Integer)
    
    On Error GoTo Proc_Err

    Dim llLen As Long
    Dim lSeq As Long
    Dim lli As Long
    Dim llLastPos As Long
    Dim lbIsDoubleQuoteOpen As Boolean
    Dim lsChar As String
    
    ' Get the length of the Line
    llLen = Len(pLine)
    
    lbIsDoubleQuoteOpen = False
    For lli = 1 To llLen
        lsChar = Mid$(pLine, lli, 1)
        Select Case Asc(lsChar)
            Case 34 ' "
                If lbIsDoubleQuoteOpen = False Then
                    ' You just opened a double-quote Setup for first time.
                    ' Save our position
                    llLastPos = lli
                    ' Setup for next time
                    lbIsDoubleQuoteOpen = True
                Else
                    ' You Found a second double quote in a row
                    If lli - 1 = llLastPos Then
                        ' If we end now then we would not have an empty range, so increase by one
                        llLastPos = lli
                    Else
                        If pMethod = 3 Or pMethod = 2 Then
                            ' Normal case, add the range in (PosOfFirst",PosOfSecond")
                            PositionsCollection.Add "S" & CStr(llLastPos + 1) & "," & CStr(lli - 1), CStr(lSeq)
                            ' Update the Key
                            lSeq = lSeq + 1
                        End If
                        ' Setup for next time
                        lbIsDoubleQuoteOpen = False
                    End If
                End If
            Case 39 ' '
                If lbIsDoubleQuoteOpen = False Then
                    If pMethod = 3 Or pMethod = 1 Then
                        ' Normal case, we do not have a string open
                        PositionsCollection.Add "C" & CStr(lli + 1) & "," & CStr(llLen), CStr(lSeq)
                        lSeq = lSeq + 1
                        ' The rest of the code is a comment - so exit the loop
                    End If
                    Exit For
                Else
                    ' We found our comment char in a string - so ignore it
                End If
        End Select
    Next
    
Proc_Exit:
    Exit Sub

Proc_Err:
    utlErrorHandler "GetPositionsToCheckInLine"
    Resume Proc_Exit
End Sub

'-------------------------------------------------------------------------------
'
' GetGivenLineOfCode - Get the given Line of Code
'
' Parameters:
'   pLine - I: Line to get
'
' Returns:
'   String. Line of code or "" if that line doesn't exist
'
' Comments:
'   None.
'
'-------------------------------------------------------------------------------
Public Function GetGivenLineOfCode(ByVal pLine As Long) As String
    On Error GoTo Proc_Err
    If oVBE.ActiveCodePane Is Nothing Then
        ' There is no ActiveCodePane so return blank
        GetGivenLineOfCode = ""
    Else
        ' Get the lines into a string
        With oVBE.ActiveCodePane.CodeModule
            ' Get the Total Number of Lines in the CodeModule
            If pLine <= .CountOfLines Or pLine > 0 Then
                GetGivenLineOfCode = .Lines(pLine, 1)
            End If
        End With
    End If
Proc_Exit:
    Exit Function
Proc_Err:
    utlErrorHandler "GetGivenLineOfCode"
    Resume Proc_Exit
End Function

'-------------------------------------------------------------------------------
'
' GetCoordsAndStringFromPos - Extract a string & other info based on pCoordStr
'
' Parameters:
'   pLine       -   I: Line to extract from
'   pCoordStr   -   I: String in form {C|S}x,y
'   pFirstPos   -   O: The x as a long
'   pLastPos    -   O: The y as a long
'   pIsComments -   O: If its a 'C' True, otherwise it must be 'S' so use False
'
' Returns:
'   String in range of pCoordStr
'
' Comments:
'   = See GetPositionsToCheckInLine() to understand what pCoordStr contains.
'   = Bad CoordStr are silently ignored.
'
'-------------------------------------------------------------------------------
Public Function GetCoordsAndStringFromPos(ByVal pLine As String, _
    ByVal pCoordStr As String, ByRef pFirstPos As Long, ByRef pLastPos As Long, _
    ByRef pIsComments As Boolean) As String
    On Error GoTo Proc_Err
    Dim lLen As Long
    Dim lPos As Long
    Dim lsValue1 As String
    Dim lsValue2 As String
    ' Get the length of a CoordStr
    lLen = Len(pCoordStr)
    If lLen < 1 Then GoTo Proc_Exit
    ' The first char is a C or an S
    pIsComments = (Left$(pCoordStr, 1) = "C")
    ' Point to the comma
    lPos = InStr(2, pCoordStr, ",")
    If lPos < 0 Then GoTo Proc_Exit
    ' Get the StartPos
    lsValue1 = Trim$(Mid$(pCoordStr, 2, lPos - 2))
    If Not IsNumeric(lsValue1) Then GoTo Proc_Err
    pFirstPos = CLng(lsValue1)
    ' Get the EndPos
    lsValue2 = Trim$(Mid$(pCoordStr, lPos + 1))
    If Not IsNumeric(lsValue2) Then GoTo Proc_Err
    pLastPos = CLng(lsValue2)
    ' Check the range is possible
    If pFirstPos > pLastPos Then
        ' Strange. It happens it cases where ' is on its own line
        pFirstPos = 0
        pLastPos = 0
        GetCoordsAndStringFromPos = ""
        GoTo Proc_Exit
    End If
    ' Get that sub string
    GetCoordsAndStringFromPos = Mid$(pLine, pFirstPos, pLastPos - pFirstPos + 1)
Proc_Exit:
    Exit Function
Proc_Err:
    utlErrorHandler "GetCoordsAndStringFromPos"
    Resume Proc_Exit
End Function
