VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsString"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'*******************************************************************************
'
'  Spell Check VB Addin - Spell Check Comments and Strings in VB code.
'
'  Module:  clsstring.cls
'  Author:  Dean J. Giovanelli (deangio.pa@netzero.net)
'  Date:    12/05/2000
'
'  Description:
'    Generic string functions object. Designed for re-use in other projects.
'    If you reuse this class elsewhere remember to create utlErrorHandler()
'
'  Modifications:
'   Help hungry children: How you treat the poorest - is how you treat God.
'*******************************************************************************
Option Explicit
' String of characters that are treated as delimiters in TokenizeStr()
Private msUserDelim As String

'-------------------------------------------------------------------------------
'
' TokenizeStr - Better generic version of vb6 split() function.
'               It provides a one-dimensional dynamic string array of words and
'               a corresponding one-dimensional dynamic string array of Positions
'               that these words exist in the string.
'               Further characters 0-32 and 127 are treated as delimiters & will
'               not appear in any words. Also any characters in msUserDelim are also
'               treated as delimiters.
'
' Parameters:
'   psString        -   I: Source String to split apart
'   plPositions     -   O: Array of positions in psString (1-based) corresponding
'                          to psWords. Caller must later free with Erase
'   psWords         -   O: Array of strings split out from vaString
'                          Caller must later free with Erase
'   plNumberOfWords -   O: # of items in psWords. E.g. Use For i = 1 to plNumberOfWords ...
'   plStartPos      -   I: (optional) pos to start looking in string. Note 1=first char
'                           defaults to 1
'   plEndPos        -   I: (optional) last pos to include within string.
'                           defaults to last char in psString
'
' Returns:
'   True=ok, False=Error
'
' Comments:
'   = Users must call erase on plPositions and psWords which are dynamic arrays
'   = E.g.
'    Dim pIndexes() As Long
'    Dim tokens() As String
'    Dim n As Long
'    Dim i As Long
'
'    If TokenizeStr("<FONT>Arial 10 Bold Strike", pIndexes(), tokens(), n, 7, 14) then
'       For i = 1 To n
'           Debug.Print "Str=" & tokens(i) & "   Pos=" & CStr(pIndexes(i))
'       Next
'       Erase pIndexes
'       Erase tokens
'    End If
'    ==> Str=Arial   Pos=7
'    ==> Str=10   Pos=13

'-------------------------------------------------------------------------------
Public Function TokenizeStr(psString As String, plPositions() As Long, _
    psWords() As String, plNumberOfWords As Long, Optional plStartPos As Long = 1, _
    Optional plEndPos As Long = 0) As Boolean

    On Error GoTo Proc_Err
    Dim llsize As Long          ' current element in arrays
    Dim lsThisWord As String    ' current word being formed
    Dim llStart As Long         ' starting position in psString to searching from
    Dim llEnd As Long           ' last position in the psString to search
    Dim llLenOfStr As Long      ' length of psString
    Dim llLoop As Long
    Dim lsChar As String
    
    ' Init
    llsize = 0
    lsThisWord = ""
    llLenOfStr = Len(psString)
    
    ' perfectly valid cases that are not errors
    If llLenOfStr = 0 Then GoTo Proc_Exit               ' no string so no work
    If plEndPos > llLenOfStr Then GoTo Proc_Exit        ' no strings could be found
    If plStartPos > llLenOfStr Then GoTo Proc_Exit      ' no strings could be found
    
    If plStartPos < 1 Then plStartPos = 1               ' assume they want to start at one
    If plEndPos < 1 Then plEndPos = llLenOfStr          ' assume they want the whole string
    
    ' Setup character range
    llStart = plStartPos
    llEnd = plEndPos
    
    ' Loop through the String
    For llLoop = llStart To llEnd
        lsChar = Mid$(psString, llLoop, 1)
        
        If Len(msUserDelim) > 0 Then
            If InStr(1, msUserDelim, lsChar) > 0 Then
                lsChar = " "
            End If
        End If
        
        Select Case Asc(lsChar)
            Case 0 To 32, 127
                ' Its invalid - so return it
                If Len(lsThisWord) = 0 Then
                    ' Do nothing
                Else
                    ' We have completed this word
                    ReDim Preserve psWords(llsize)
                    psWords(llsize) = lsThisWord
                    lsThisWord = ""
                End If
            Case Else
                ' Its valid
                If Len(lsThisWord) = 0 Then
                    ' Start a new word
                    llsize = llsize + 1
                    ReDim Preserve plPositions(llsize)
                    plPositions(llsize) = llLoop ' Preserve the index
                    lsThisWord = lsThisWord + lsChar
                Else
                    ' We found another legal character - so use it
                    lsThisWord = lsThisWord + lsChar
                End If
        End Select
    Next
    If lsThisWord <> "" Then ' we have to finish our word
        ' We have completed the last word
        ReDim Preserve psWords(llsize)
        psWords(llsize) = lsThisWord
        lsThisWord = ""
    End If
    plNumberOfWords = llsize
    TokenizeStr = True
    
Proc_Exit:
    Exit Function

Proc_Err:
    utlErrorHandler "TokenizeStr"
    Resume Proc_Exit
End Function

'-------------------------------------------------------------------------------
'
' StripCtrlCharsInStr - Removes all control characters from string
'
' Parameters:
'   I: pCodeLine: Source string
'
' Returns:
'   boolean: True it worked, False failed.
'
' Comments:
'   Example: Dim aStr as String
'            aStr = StripCtrlCharsInStr( _ <--
'             "The" & vbtab & vbCrLf & "End" & vbCrLf)
'             Result--> "TheEnd"
'
'-------------------------------------------------------------------------------
Public Function StripCtrlCharsInStr(ByVal pCodeLine As String) As String
    On Error GoTo Proc_Err
    Dim llLoop As Long
    Dim llLenOfStr As Long
    Dim lsChar As String
    Dim lsTarget As String
    '
    llLenOfStr = Len(pCodeLine)
    If llLenOfStr <> 0 Then
        For llLoop = 1 To llLenOfStr
            lsChar = Mid$(pCodeLine, llLoop, 1)
            Select Case Asc(lsChar)
                Case 0 To 31, 127
                    ' Do nothing
                Case Else
                    lsTarget = lsTarget & lsChar
            End Select
        Next
    End If
    ' We didn't find one
    StripCtrlCharsInStr = lsTarget
Proc_Exit:
    Exit Function
Proc_Err:
    utlErrorHandler "StripCtrlCharsInStr"
    Resume Proc_Exit
End Function

'-------------------------------------------------------------------------------
'
' UserDelim - Get the current delimiter characters
'
' Parameters:
'   None.
'
' Returns:
'   None.
'
' Comments:
'   = These characters are used by TokenizeStr()
'
'-------------------------------------------------------------------------------
Public Property Get UserDelim() As String
    On Error GoTo Proc_Err
    UserDelim = msUserDelim
Proc_Exit:
    Exit Property
Proc_Err:
    utlErrorHandler "UserDelim"
    Resume Proc_Exit
End Property

'-------------------------------------------------------------------------------
'
' UserDelim - Set the current delimiter characters
'
' Parameters:
'   vNewValue -
'
' Returns:
'   None.
'
' Comments:
'   = These characters are used by TokenizeStr()
'
'-------------------------------------------------------------------------------
Public Property Let UserDelim(ByVal vNewValue As String)
    On Error GoTo Proc_Err
    msUserDelim = vNewValue
Proc_Exit:
    Exit Property
Proc_Err:
    utlErrorHandler "UserDelim"
    Resume Proc_Exit
End Property
